(function ($, Drupal, once) {
  Drupal.behaviors.aceEditor = {
    attach: function (context) {
      $(once('json-yaml-editor', '.yaml-editor', context)).each(function () {
        let editorID = $(this).data('editor');
        if(mermaid){
          mermaid.initialize({startOnLoad: true});
          window.addEventListener('load', function () {
            var svgs = d3.selectAll(".mermaid svg");
            svgs.each(function() {
              var svg = d3.select(this);
              svg.html("<g>" + svg.html() + "</g>");
              var inner = svg.select("g");
              var zoom = d3.zoom().on("zoom", function(event) {
                inner.attr("transform", event.transform);
              });
              svg.call(zoom);
            });
          });
        }
        if (ace) {
          let $textarea = $(this);
          let editor = ace.edit(editorID);

          if($textarea.val() != ''){
            editor.setValue($textarea.val());
          }
          editor.getSession().setMode( "ace/mode/yaml" );
          editor.focus();
          editor.getSession().setTabSize(2);
          editor.setTheme('ace/theme/chrome');
          editor.setOptions({
            minLines: 5,
            maxLines: 30
          });

          // Update Drupal textarea value.
          editor.getSession().on('change', function () {
            let yamlString = editor.getSession().getValue();
            $textarea.val(yamlString);

            try {
              const stateMachine = jsyaml.load(yamlString);
              let mermaidString = mermaidConvertTransition(stateMachine);
              $('#stateDiagram').text(mermaidString).removeAttr('data-processed');
              mermaid.init();
            } catch (e) {
              console.error(Drupal.t('YAML is not validate:'), e.message);
            }
          });
        }
      });
    }
  };

  function mermaidConvertTransition(stateMachine){
    const stateDiagram = ["stateDiagram-v2"];
    const defaultState = '[*]';
    const transitions = stateMachine.transitions;
    const states = stateMachine.states;

    Object.keys(states).forEach(key => {
      let state = states[key];
      let stateName = key.replace(/[^a-zA-Z0-9_*\[\]]/g, '_');
      let stateLabel = state.label.replace(/:/g, '');
      stateDiagram.push(stateName + ':' + stateLabel);
    });

    Object.keys(transitions).forEach(key => {
      const transition = transitions[key];
      if (Array.isArray(transition.from)) {
        transition.from.forEach((state) => {
          if (state == '') {
            state = defaultState;
          }
          state = state.replace(/[^a-zA-Z0-9_*\[\]]/g, '_');
          let to = transition.to.replace(/[^a-zA-Z0-9_*\[\]]/g, '_');
          if (to == '') {
            to = '[*]';
          }
          let query = `${state} --> ${to}`;
          if (transition.label) {
            query += `: ${transition.label}`;
          }
          stateDiagram.push(query);
        });
      }
    });
    return stateDiagram.join("\n");
  }
}(jQuery, Drupal, once));
